/*
 * Read/write ion storm data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

ION_STORM *read_ion_storm(int inum)
{
  FILE *handle;
  ION_STORM *str;

  handle=mkopen("grey.hst",false,'r');
  if (!handle) return NULL;

  if (inum>50 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,0x3e8+0x10*(inum-1),SEEK_SET);

  str=new ION_STORM;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->pos.x=readSB2(handle);
  str->pos.y=readSB2(handle);
  str->wRadius=readUB2(handle);
  str->wVoltage=readUB2(handle);
  str->wHeading=readUB2(handle);
  str->wGrowShrink=readUB2(handle);

  fclose(handle);
  return str;
}

ION_STORM *read_all_ion_storm()
{
  FILE *handle;
  ION_STORM *str;
  int inum;

  handle=mkopen("grey.hst",false,'r');
  if (!handle) return NULL;

  str=new ION_STORM [51];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,0x3e8,SEEK_SET);

  for(inum=1;inum<=50;inum++) {
    str[inum].pos.x=readSB2(handle);
    str[inum].pos.y=readSB2(handle);
    str[inum].wRadius=readUB2(handle);
    str[inum].wVoltage=readUB2(handle);
    str[inum].wHeading=readUB2(handle);
    str[inum].wGrowShrink=readUB2(handle);
    readUB4(handle);
  }

  fclose(handle);
  return str;
}

bool write_ion_storm(int inum,ION_STORM *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("grey.hst",true,'w');
  if (!handle) return NULL;

  if (inum>50 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,0x3e8+0x10*(inum-1),SEEK_SET);

  v=writeSB2(handle,str->pos.x);
  if (v) v=writeSB2(handle,str->pos.y);
  if (v) v=writeUB2(handle,str->wRadius);
  if (v) v=writeUB2(handle,str->wVoltage);
  if (v) v=writeUB2(handle,str->wHeading);
  if (v) v=writeUB2(handle,str->wGrowShrink);

  fclose(handle);
  return v;
}

ION_SHIPEX *read_ion_shipex(int snum)
{
  FILE *handle;
  ION_SHIPEX *str;

  handle=mkopen("grey.hst",false,'r');
  if (!handle) return NULL;

  if (snum>MAXSHIP || snum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2*(snum-1),SEEK_SET);

  str=new ION_SHIPEX;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wExperience=readUB2(handle);

  fclose(handle);
  return str;
}

ION_SHIPEX *read_all_ion_shipex()
{
  FILE *handle;
  ION_SHIPEX *str;
  int snum;

  handle=mkopen("grey.hst",false,'r');
  if (!handle) return NULL;

  str=new ION_SHIPEX [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  for(snum=1;snum<501;snum++) {
    str[snum].wExperience=readUB2(handle);
  }

  fclose(handle);
  return str;
}

bool write_ion_shipex(int snum,ION_SHIPEX *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("grey.hst",true,'w');
  if (!handle) return NULL;

  if (snum>MAXSHIP || snum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2*(snum-1),SEEK_SET);

  v=writeSB2(handle,str->wExperience);

  fclose(handle);
  return v;
}

ION_NAME *read_ion_name(int inum)
{
  FILE *handle;
  ION_NAME *str;

  handle=mkopen("storm.nm",true,'r');
  if (!handle) return NULL;

  if (inum>50 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,20*(inum-1),SEEK_SET);

  str=new ION_NAME;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->sName=readXString(handle,20);

  fclose(handle);
  return str;
}

bool write_ion_name(int inum,ION_NAME *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("storm.nm",true,'w');
  if (!handle) return NULL;

  if (inum>50 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,20*(inum-1),SEEK_SET);

  v=writeXString(handle,20,str->sName);

  fclose(handle);
  return v;
}

ION_NAME *read_all_ion_name()
{
  FILE *handle;
  ION_NAME *str;
  int inum;

  handle=mkopen("storm.nm",true,'r');
  if (!handle) return NULL;

  str=new ION_NAME [51];

  for (inum=1;inum<=50;inum++) {
  str[inum].sName=readXString(handle,20);
  }

  fclose(handle);
  return str;
}
