/*
 * Write hull-list and *spec data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

bool write_truehull(int hnum,int playernum,TRUEHULL *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("truehull.dat",true,'w');
  if (!handle) return false;

  if (hnum>20 || hnum<1) {
    fclose(handle);
    return false;
  }

  fseek(handle,2*(hnum-1)+40*(playernum-1),SEEK_SET);

  v=writeUB2(handle,str->wHullnum);

  fclose(handle);
  return v;
}

bool write_hullspec(int hnum,HULLSPEC *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("hullspec.dat",true,'w');
  if (!handle) return NULL;

  if (hnum>MAXHULL || hnum<1) {
    fclose(handle);
    return false;
  }

  fseek(handle,60*(hnum-1),SEEK_SET);

  v=writeXString(handle,30,str->sName);
  if (v) v=writeUB2(handle,str->wPicture);
  if (v) v=writeUB2(handle,str->wDamagePic);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->wFuel);
  if (v) v=writeUB2(handle,str->wCrew);
  if (v) v=writeUB2(handle,str->wEngines);
  if (v) v=writeUB2(handle,str->wMass);
  if (v) v=writeUB2(handle,str->wTechLevel);
  if (v) v=writeUB2(handle,str->wCargo);
  if (v) v=writeUB2(handle,str->wFighterBays);
  if (v) v=writeUB2(handle,str->wTorpsBays);
  if (v) v=writeUB2(handle,str->wBeams);
  if (v) v=writeUB2(handle,str->cargo.money);

  fclose(handle);
  return v;
}

bool write_beamspec(int bnum,BEAMSPEC *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("beamspec.dat",true,'w');
  if (!handle) return false;

  if (bnum>10) {
    fclose(handle);
    return false;
  }

  fseek(handle,36*(bnum-1),SEEK_SET);

  v=writeXString(handle,20,str->sName);
  if (v) v=writeUB2(handle,str->cargo.money);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->wMass);
  if (v) v=writeUB2(handle,str->wTechLevel);
  if (v) v=writeUB2(handle,str->wKill);
  if (v) v=writeUB2(handle,str->wExplosive);

  fclose(handle);
  return v;
}

bool write_torpspec(int tnum,TORPSPEC *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("torpspec.dat",true,'w');
  if (!handle) return false;

  if (tnum>10) {
    fclose(handle);
    return false;
  }

  fseek(handle,38*(tnum-1),SEEK_SET);

  v=writeXString(handle,20,str->sName);
  if (v) v=writeUB2(handle,str->wCostTorp);
  if (v) v=writeUB2(handle,str->cargo.money);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->wMass);
  if (v) v=writeUB2(handle,str->wTechLevel);
  if (v) v=writeUB2(handle,str->wKill);
  if (v) v=writeUB2(handle,str->wExplosive);

  fclose(handle);
  return v;
}

bool write_engspec(int e_num,ENGSPEC *str)
{
  FILE *handle;
  bool v;
  int k;

  handle=mkopen("engspec.dat",true,'w');
  if (!handle) return false;

  if (e_num>9) {
    fclose(handle);
    return false;
  }

  fseek(handle,66*(e_num-1),SEEK_SET);

  v=writeXString(handle,20,str->sName);
  if (v) v=writeUB2(handle,str->cargo.money);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->wTechLevel);
  for (k=1;k<10;k++) {
    if (v) v=writeUB4(handle,str->lFuelUse[k]);
  }

  fclose(handle);
  return v;
}

bool make_checksum()
{
  UB4 cs;
  UB1 cw;
  fpos_t k=0;
  FILE *handle;
  bool v;

  cs=make_cs("hullspec.dat",k);
  cs+=make_cs("engspec.dat",k);
  cs+=make_cs("beamspec.dat",k);
  k=0x17d;
  cs+=make_cs("torpspec.dat",k);

  if (cs > 0x52B81) return false;
  if (cs < 0x4D1DB) return false;
  cs=0x52B81-cs;
  handle=mkopen("torpspec.dat",true,'w');
  if (!handle) return false;
  v=true;
  fseek(handle,k,SEEK_SET);

  for (;cs>0;k++) {
    if (cs>0xff) {
      cw=0xff;
      cs-=0xff;
    } else {
      cw=cs;
      cs=0;
    }
    if (v) v=writeUB1(handle,cw);
  }
  cw=0;
  if (k<0x1d6) {
    for (;k<0x1d6;k++) {
      if (v) v=writeUB1(handle,cw);
    }
  }

  fclose(handle);
  return v;
}

UB4 make_cs(const char *filename,fpos_t len)
{
  UB4 cs=0;
  fpos_t pos;
  FILE *handle;

  handle=mkopen(filename,true,'r');
  if (len==0) {
    fseek(handle,0,SEEK_END);
    len=ftell(handle);
  }

  fseek(handle,0,SEEK_SET);
  for (pos=0;pos<len;pos++) {
    cs += readUB1(handle);
  }
  fclose(handle);

  return cs;
}
