/*
 * Read/write planet data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

PLANETDATA *read_planetdata(int pnum)
{
  FILE *handle;
  PLANETDATA *str;

  handle=mkopen("pdata.hst",false,'r');
  if (!handle) return NULL;

  if (pnum>NUM_PLANETS || pnum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+85*(pnum-1),SEEK_SET);

  str=new PLANETDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wOwner=readUB2(handle);
  str->wID=readUB2(handle);
  str->sCode=readXString(handle,3);
  str->wMines=readUB2(handle);
  str->wFactories=readUB2(handle);
  str->wDefense=readUB2(handle);
  str->cargo.fuel=readUB4(handle);
  str->cargo.tri=readUB4(handle);
  str->cargo.dur=readUB4(handle);
  str->cargo.mol=readUB4(handle);
  str->cargo.clans=readUB4(handle);
  str->cargo.supplies=readUB4(handle);
  str->cargo.money=readUB4(handle);
  str->left.fuel=readUB4(handle);
  str->left.tri=readUB4(handle);
  str->left.dur=readUB4(handle);
  str->left.mol=readUB4(handle);
  str->density.fuel=readUB2(handle);
  str->density.tri=readUB2(handle);
  str->density.dur=readUB2(handle);
  str->density.mol=readUB2(handle);
  str->wColTax=readUB2(handle);
  str->wNativeTax=readUB2(handle);
  str->iColStatus=readSB2(handle);
  str->iNativeStatus=readSB2(handle);
  str->wGov=readUB2(handle);
  str->lNatives=readUB4(handle);
  str->wRace=readUB2(handle);
  str->wTemperature=100-readUB2(handle);
  str->BuildNewBase=readUB2(handle);

  if (extraplan) {

    PLANET_COORDS *plc;
    plc=read_planet_coords(str->wID);
    str->pos=plc->pos;
    delete plc;

    PLANET_NAME *pn;
    pn=read_planet_name(str->wID);
    str->sName=pn->sName;
    delete pn;

  }

  fclose(handle);
  return str;
}

bool write_planetdata(int pnum,PLANETDATA *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("pdata.hst",false,'w');
  if (!handle) return false;

  if (pnum>NUM_PLANETS || pnum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+85*(pnum-1),SEEK_SET);

  v=writeUB2(handle,str->wOwner);
  if (v) v=writeUB2(handle,str->wID);
  if (v) v=writeXString(handle,3,str->sCode);
  if (v) v=writeUB2(handle,str->wMines);
  if (v) v=writeUB2(handle,str->wFactories);
  if (v) v=writeUB2(handle,str->wDefense);
  if (v) v=writeUB4(handle,str->cargo.fuel);
  if (v) v=writeUB4(handle,str->cargo.tri);
  if (v) v=writeUB4(handle,str->cargo.dur);
  if (v) v=writeUB4(handle,str->cargo.mol);
  if (v) v=writeUB4(handle,str->cargo.clans);
  if (v) v=writeUB4(handle,str->cargo.supplies);
  if (v) v=writeUB4(handle,str->cargo.money);
  if (v) v=writeUB4(handle,str->left.fuel);
  if (v) v=writeUB4(handle,str->left.tri);
  if (v) v=writeUB4(handle,str->left.dur);
  if (v) v=writeUB4(handle,str->left.mol);
  if (v) v=writeUB2(handle,str->density.fuel);
  if (v) v=writeUB2(handle,str->density.tri);
  if (v) v=writeUB2(handle,str->density.dur);
  if (v) v=writeUB2(handle,str->density.mol);
  if (v) v=writeUB2(handle,str->wColTax);
  if (v) v=writeUB2(handle,str->wNativeTax);
  if (v) v=writeSB2(handle,str->iColStatus);
  if (v) v=writeSB2(handle,str->iNativeStatus);
  if (v) v=writeUB2(handle,str->wGov);
  if (v) v=writeUB4(handle,str->lNatives);
  if (v) v=writeUB2(handle,str->wRace);
  if (v) v=writeUB2(handle,100-str->wTemperature);
  if (v) v=writeUB2(handle,str->BuildNewBase);

  fclose(handle);
  return v;
}

PLANETDATA * read_all_planetdata()
{
  FILE *handle;
  PLANETDATA *str;
  int pnum;
  PLANET_COORDS *plc;
  PLANET_NAME *pln;

  handle=mkopen("pdata.hst",false,'r');
  if (!handle) return NULL;

  str=new PLANETDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  plc=read_all_planet_coords();
  pln=read_all_planet_name();

  fseek(handle,2,SEEK_SET);

  for (pnum=1;pnum<=500;pnum++) {
    str[pnum].wOwner=readUB2(handle);
    str[pnum].wID=readUB2(handle);
    str[pnum].sCode=readXString(handle,3);
    str[pnum].wMines=readUB2(handle);
    str[pnum].wFactories=readUB2(handle);
    str[pnum].wDefense=readUB2(handle);
    str[pnum].cargo.fuel=readUB4(handle);
    str[pnum].cargo.tri=readUB4(handle);
    str[pnum].cargo.dur=readUB4(handle);
    str[pnum].cargo.mol=readUB4(handle);
    str[pnum].cargo.clans=readUB4(handle);
    str[pnum].cargo.supplies=readUB4(handle);
    str[pnum].cargo.money=readUB4(handle);
    str[pnum].left.fuel=readUB4(handle);
    str[pnum].left.tri=readUB4(handle);
    str[pnum].left.dur=readUB4(handle);
    str[pnum].left.mol=readUB4(handle);
    str[pnum].density.fuel=readUB2(handle);
    str[pnum].density.tri=readUB2(handle);
    str[pnum].density.dur=readUB2(handle);
    str[pnum].density.mol=readUB2(handle);
    str[pnum].wColTax=readUB2(handle);
    str[pnum].wNativeTax=readUB2(handle);
    str[pnum].iColStatus=readSB2(handle);
    str[pnum].iNativeStatus=readSB2(handle);
    str[pnum].wGov=readUB2(handle);
    str[pnum].lNatives=readUB4(handle);
    str[pnum].wRace=readUB2(handle);
    str[pnum].wTemperature=100-readUB2(handle);
    str[pnum].BuildNewBase=readUB2(handle);

    if (extraplan) {
      str[pnum].pos=plc[pnum].pos;
      str[pnum].sName=pln[pnum].sName;
    }
  }

  if (extraplan) {
    delete[] plc;
    delete[] pln;
  }

  fclose(handle);
  return str;
}
