/*
 * Read/write planet data
 * part of the Player-aToolkit package
 */

#include <stdio.h>
#include "tkplayer.h"
#include "tkp.h"
#include "tkiocomm.h"

static UB2 planetindex[500];
static bool planetabs;

int num_planets(bool datfile)
{
  FILE *handle;
  int ns;

  if (datfile) {
    handle=mkopen("pdata&p.dat",false,'r');
  } else {
    handle=mkopen("pdata&p.dis",false,'r');
  }
  if (!handle) return 0;

  ns=(int)readUB2(handle);
  fclose(handle);
  return ns;
}

PLANETDATA *read_planetdata(int pnum,bool datfile)
{
  if (planetabs==false) {
    PLANETDATA *tstr;
    int s,k;
    k=num_planets();
    for(s=1;s<=k;s++) {
      tstr=read_planetdata_A(s,datfile);
      if (tstr) {
        planetindex[tstr->wID-1]=s;
        delete tstr;
      }
    }
    planetabs=true;
  }

  return read_planetdata_A(planetindex[pnum-1],datfile);
}


PLANETDATA *read_planetdata_A(int pnum,bool datfile)
{
  FILE *handle;
  PLANETDATA *str;
  UB2 ns;

  if (datfile) {
    handle=mkopen("pdata&p.dat",false,'r');
  } else {
    handle=mkopen("pdata&p.dis",false,'r');
  }
  if (!handle) return NULL;

  ns=readUB2(handle);
  if (pnum>ns || pnum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+85*(pnum-1),SEEK_SET);

  str=new PLANETDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wOwner=readUB2(handle);
  str->wID=readUB2(handle);
  str->sCode=readXString(handle,3);
  str->wMines=readUB2(handle);
  str->wFactories=readUB2(handle);
  str->wDefense=readUB2(handle);
  str->cargo.fuel=readUB4(handle);
  str->cargo.tri=readUB4(handle);
  str->cargo.dur=readUB4(handle);
  str->cargo.mol=readUB4(handle);
  str->cargo.clans=readUB4(handle);
  str->cargo.supplies=readUB4(handle);
  str->cargo.money=readUB4(handle);
  str->left.fuel=readUB4(handle);
  str->left.tri=readUB4(handle);
  str->left.dur=readUB4(handle);
  str->left.mol=readUB4(handle);
  str->density.fuel=readUB2(handle);
  str->density.tri=readUB2(handle);
  str->density.dur=readUB2(handle);
  str->density.mol=readUB2(handle);
  str->wColTax=readUB2(handle);
  str->wNativeTax=readUB2(handle);
  str->iColStatus=readSB2(handle);
  str->iNativeStatus=readSB2(handle);
  str->wGov=readUB2(handle);
  str->lNatives=readUB4(handle);
  str->wRace=readUB2(handle);
  str->wTemperature=100-readUB2(handle);
  str->BuildNewBase=readUB2(handle);

  fclose(handle);

  if (extraplan) {

    PLANET_COORDS *plc;
    plc=read_planet_coords(str->wID);
    str->pos=plc->pos;
    delete plc;

    PLANET_NAME *pn;
    pn=read_planet_name(str->wID);
    str->sName=pn->sName;
    delete pn;

  }

  return str;
}

bool write_planetdata(int snum,PLANETDATA *str)
{
  if (planetabs==false) {
    PLANETDATA *tstr;
    tstr=read_planetdata(1);
    if (tstr) delete tstr;
  }

  return write_planetdata_A(planetindex[snum-1],str);
}

bool write_planetdata_A(int pnum,PLANETDATA *str)
{
  FILE *handle;
  UB2 ns;
  bool v;

  handle=mkopen("pdata&p.dat",false,'w');
  if (!handle) return false;

  ns=readUB2(handle);
  if (pnum>ns || pnum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+85*(pnum-1),SEEK_SET);

  v=writeUB2(handle,str->wOwner);
  if (v) v=writeUB2(handle,str->wID);
  if (v) v=writeXString(handle,3,str->sCode);
  if (v) v=writeUB2(handle,str->wMines);
  if (v) v=writeUB2(handle,str->wFactories);
  if (v) v=writeUB2(handle,str->wDefense);
  if (v) v=writeUB4(handle,str->cargo.fuel);
  if (v) v=writeUB4(handle,str->cargo.tri);
  if (v) v=writeUB4(handle,str->cargo.dur);
  if (v) v=writeUB4(handle,str->cargo.mol);
  if (v) v=writeUB4(handle,str->cargo.clans);
  if (v) v=writeUB4(handle,str->cargo.supplies);
  if (v) v=writeUB4(handle,str->cargo.money);
  if (v) v=writeUB4(handle,str->left.fuel);
  if (v) v=writeUB4(handle,str->left.tri);
  if (v) v=writeUB4(handle,str->left.dur);
  if (v) v=writeUB4(handle,str->left.mol);
  if (v) v=writeUB2(handle,str->density.fuel);
  if (v) v=writeUB2(handle,str->density.tri);
  if (v) v=writeUB2(handle,str->density.dur);
  if (v) v=writeUB2(handle,str->density.mol);
  if (v) v=writeUB2(handle,str->wColTax);
  if (v) v=writeUB2(handle,str->wNativeTax);
  if (v) v=writeSB2(handle,str->iColStatus);
  if (v) v=writeSB2(handle,str->iNativeStatus);
  if (v) v=writeUB2(handle,str->wGov);
  if (v) v=writeUB4(handle,str->lNatives);
  if (v) v=writeUB2(handle,str->wRace);
  if (v) v=writeUB2(handle,100-str->wTemperature);
  if (v) v=writeUB2(handle,str->BuildNewBase);

  fclose(handle);
  return v;
}

PLANETDATA * read_all_planetdata(bool datfile)
{
  FILE *handle;
  PLANETDATA *str;
  int k,pnum,pmax;
  UB2 pid,tmpown;
  PLANET_COORDS *plc;
  PLANET_NAME *pln;

  pmax=num_planets(datfile);
  if (pmax==0) return NULL;

  if (datfile) {
    handle=mkopen("pdata&p.dat",false,'r');
  } else {
    handle=mkopen("pdata&p.dis",false,'r');
  }
  if (!handle) return NULL;

  str=new PLANETDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  plc=read_all_planet_coords();
  pln=read_all_planet_name();

  for(k=1;k<501;k++) {
    str[k].wID=0;
  }

  fseek(handle,2,SEEK_SET);

  for (pnum=1;pnum<=pmax;pnum++) {
    tmpown=readUB2(handle);
    pid=readUB2(handle);
    str[pid].wOwner=tmpown;
    str[pid].wID=pid;
    str[pid].sCode=readXString(handle,3);
    str[pid].wMines=readUB2(handle);
    str[pid].wFactories=readUB2(handle);
    str[pid].wDefense=readUB2(handle);
    str[pid].cargo.fuel=readUB4(handle);
    str[pid].cargo.tri=readUB4(handle);
    str[pid].cargo.dur=readUB4(handle);
    str[pid].cargo.mol=readUB4(handle);
    str[pid].cargo.clans=readUB4(handle);
    str[pid].cargo.supplies=readUB4(handle);
    str[pid].cargo.money=readUB4(handle);
    str[pid].left.fuel=readUB4(handle);
    str[pid].left.tri=readUB4(handle);
    str[pid].left.dur=readUB4(handle);
    str[pid].left.mol=readUB4(handle);
    str[pid].density.fuel=readUB2(handle);
    str[pid].density.tri=readUB2(handle);
    str[pid].density.dur=readUB2(handle);
    str[pid].density.mol=readUB2(handle);
    str[pid].wColTax=readUB2(handle);
    str[pid].wNativeTax=readUB2(handle);
    str[pid].iColStatus=readSB2(handle);
    str[pid].iNativeStatus=readSB2(handle);
    str[pid].wGov=readUB2(handle);
    str[pid].lNatives=readUB4(handle);
    str[pid].wRace=readUB2(handle);
    str[pid].wTemperature=100-readUB2(handle);
    str[pid].BuildNewBase=readUB2(handle);

    if (extraplan) {
      str[pid].pos=plc[pid].pos;
      str[pid].sName=pln[pid].sName;
    }
  }

  if (extraplan) {
    delete[] plc;
    delete[] pln;
  }

  fclose(handle);
  return str;
}
