/*
 * Read/write ship data
 * part of the Player-Toolkit package
 */

#include <stdio.h>
#include "tkplayer.h"
#include "tkp.h"
#include "tkiocomm.h"

static UB2 shipindex[500];
static bool shipabs;

int num_ships(bool datfile)
{
  FILE *handle;
  int ns;

  if (datfile) {
    handle=mkopen("ship&p.dat",false,'r');
  } else {
    handle=mkopen("ship&p.dis",false,'r');
  }
  if (!handle) return 0;

  ns=(int)readUB2(handle);
  fclose(handle);
  return ns;
}

SHIPDATA *read_shipdata(int snum,bool datfile)
{
  if (shipabs==false) {
    SHIPDATA *tstr;
    int s,k;
    k=num_ships();
    for(s=1;s<=k;s++) {
      tstr=read_shipdata_A(s,datfile);
      if (tstr) {
        shipindex[tstr->wID-1]=s;
        delete tstr;
      }
    }
    shipabs=true;
  }

  return read_shipdata_A(shipindex[snum-1],datfile);
}

SHIPDATA *read_shipdata_A(int snum,bool datfile)
{
  FILE *handle;
  SHIPDATA *str;
  UB2 ns;

  if (datfile) {
    handle=mkopen("ship&p.dat",false,'r');
  } else {
    handle=mkopen("ship&p.dis",false,'r');
  }
  if (!handle) return NULL;

  ns=readUB2(handle);
  if (snum>ns || snum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+107*(snum-1),SEEK_SET);

  str=new SHIPDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wID=readUB2(handle);
  str->wRace=readUB2(handle);
  str->sCode=readXString(handle,3);
  str->wWarp=readUB2(handle);
  str->dis.x=readSB2(handle);
  str->dis.y=readSB2(handle);
  str->pos.x=readUB2(handle);
  str->pos.y=readUB2(handle);
  str->wDriveLevel=readUB2(handle);
  str->wShipType=readUB2(handle);
  str->wBeamLevel=readUB2(handle);
  str->wBeamWeapons=readUB2(handle);
  str->wFighterBays=readUB2(handle);
  str->wTorpLevel=readUB2(handle);
  str->wTorps=readUB2(handle);
  str->wTorpsBays=readUB2(handle);
  str->wMission=readUB2(handle);
  str->wEnemy=readUB2(handle);
  str->wTowed=readUB2(handle);
  str->wDamage=readUB2(handle);
  str->wCrew=readUB2(handle);
  str->cargo.clans=readUB2(handle);
  str->sName=readXString(handle,20);
  str->cargo.fuel=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->cargo.supplies=readUB2(handle);
  str->toplanet.fuel=readUB2(handle);
  str->toplanet.tri=readUB2(handle);
  str->toplanet.dur=readUB2(handle);
  str->toplanet.mol=readUB2(handle);
  str->toplanet.clans=readUB2(handle);
  str->toplanet.supplies=readUB2(handle);
  str->wPID=readUB2(handle);
  str->toship.fuel=readUB2(handle);
  str->toship.tri=readUB2(handle);
  str->toship.dur=readUB2(handle);
  str->toship.mol=readUB2(handle);
  str->toship.clans=readUB2(handle);
  str->toship.supplies=readUB2(handle);
  str->wSID=readUB2(handle);
  str->wIntercept=readUB2(handle);
  str->cargo.money=readUB2(handle);

  if (extraship) {
    HULLSPEC *hs;
    hs=read_hullspec(str->wShipType);
    str->hullspec=*hs;
    delete hs;
    ENGSPEC *es;
    es=read_engspec(str->wDriveLevel);
    str->engspec=*es;
    delete es;
    if (str->wBeamWeapons > 0) {
      BEAMSPEC *bs;
      bs=read_beamspec(str->wBeamLevel);
      str->beamspec=*bs;
      delete bs;
    }
    if (str->wTorpsBays > 0) {
      TORPSPEC *ts;
      ts=read_torpspec(str->wTorpLevel);
      str->torpspec=*ts;
      delete ts;
    }
  }

  fclose(handle);
  return str;
}

bool write_shipdata(int snum,SHIPDATA *str)
{
  if (shipabs==false) {
    SHIPDATA *tstr;
    tstr=read_shipdata(1);
    if (tstr) delete tstr;
  }

  return write_shipdata_A(shipindex[snum-1],str);
}

bool write_shipdata_A(int snum,SHIPDATA *str)
{
  FILE *handle;
  UB2 ns;
  bool v;

  handle=mkopen("ship&p.dat",false,'w');
  if (!handle) return false;

  ns=readUB2(handle);
  if (snum>ns || snum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+107*(snum-1),SEEK_SET);

  v=writeUB2(handle,str->wID);
  if (v) v=writeUB2(handle,str->wRace);
  if (v) v=writeXString(handle,3,str->sCode);
  if (v) v=writeUB2(handle,str->wWarp);
  if (v) v=writeSB2(handle,str->dis.x);
  if (v) v=writeSB2(handle,str->dis.y);
  if (v) v=writeUB2(handle,str->pos.x);
  if (v) v=writeUB2(handle,str->pos.y);
  if (v) v=writeUB2(handle,str->wDriveLevel);
  if (v) v=writeUB2(handle,str->wShipType);
  if (v) v=writeUB2(handle,str->wBeamLevel);
  if (v) v=writeUB2(handle,str->wBeamWeapons);
  if (v) v=writeUB2(handle,str->wFighterBays);
  if (v) v=writeUB2(handle,str->wTorpLevel);
  if (v) v=writeUB2(handle,str->wTorps);
  if (v) v=writeUB2(handle,str->wTorpsBays);
  if (v) v=writeUB2(handle,str->wMission);
  if (v) v=writeUB2(handle,str->wEnemy);
  if (v) v=writeUB2(handle,str->wTowed);
  if (v) v=writeUB2(handle,str->wDamage);
  if (v) v=writeUB2(handle,str->wCrew);
  if (v) v=writeUB2(handle,str->cargo.clans);
  if (v) v=writeXString(handle,20,str->sName);
  if (v) v=writeUB2(handle,str->cargo.fuel);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->cargo.supplies);
  if (v) v=writeUB2(handle,str->toplanet.fuel);
  if (v) v=writeUB2(handle,str->toplanet.tri);
  if (v) v=writeUB2(handle,str->toplanet.dur);
  if (v) v=writeUB2(handle,str->toplanet.mol);
  if (v) v=writeUB2(handle,str->toplanet.clans);
  if (v) v=writeUB2(handle,str->toplanet.supplies);
  if (v) v=writeUB2(handle,str->wPID);
  if (v) v=writeUB2(handle,str->toship.fuel);
  if (v) v=writeUB2(handle,str->toship.tri);
  if (v) v=writeUB2(handle,str->toship.dur);
  if (v) v=writeUB2(handle,str->toship.mol);
  if (v) v=writeUB2(handle,str->toship.clans);
  if (v) v=writeUB2(handle,str->toship.supplies);
  if (v) v=writeUB2(handle,str->wSID);
  if (v) v=writeUB2(handle,str->wIntercept);
  if (v) v=writeUB2(handle,str->cargo.money);

  fclose(handle);
  return v;
}

SHIPDATA * read_all_shipdata(bool datfile)
{
  FILE *handle;
  SHIPDATA *str;
  int k,snum,smax;
  UB2 sid;
  HULLSPEC *xhullspec=NULL;
  ENGSPEC *xengspec=NULL;
  BEAMSPEC *xbeamspec=NULL;
  TORPSPEC *xtorpspec=NULL;

  smax=num_ships(datfile);
  if (smax==0) return NULL;

  if (datfile) {
    handle=mkopen("ship&p.dat",false,'r');
  } else {
    handle=mkopen("ship&p.dis",false,'r');
  }
  if (!handle) return NULL;

  str=new SHIPDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  if (extraship) {
    xhullspec=read_all_hullspec();
    xengspec=read_all_engspec();
    xbeamspec=read_all_beamspec();
    xtorpspec=read_all_torpspec();
  }

  for(k=1;k<501;k++) {
    str[k].wID=0;
  }

  fseek(handle,2,SEEK_SET);

  for (snum=1;snum<=smax;snum++) {
    sid=readUB2(handle);
    str[sid].wID=sid;
    str[sid].wRace=readUB2(handle);
    str[sid].sCode=readXString(handle,3);
    str[sid].wWarp=readUB2(handle);
    str[sid].dis.x=readSB2(handle);
    str[sid].dis.y=readSB2(handle);
    str[sid].pos.x=readUB2(handle);
    str[sid].pos.y=readUB2(handle);
    str[sid].wDriveLevel=readUB2(handle);
    str[sid].wShipType=readUB2(handle);
    str[sid].wBeamLevel=readUB2(handle);
    str[sid].wBeamWeapons=readUB2(handle);
    str[sid].wFighterBays=readUB2(handle);
    str[sid].wTorpLevel=readUB2(handle);
    str[sid].wTorps=readUB2(handle);
    str[sid].wTorpsBays=readUB2(handle);
    str[sid].wMission=readUB2(handle);
    str[sid].wEnemy=readUB2(handle);
    str[sid].wTowed=readUB2(handle);
    str[sid].wDamage=readUB2(handle);
    str[sid].wCrew=readUB2(handle);
    str[sid].cargo.clans=readUB2(handle);
    str[sid].sName=readXString(handle,20);
    str[sid].cargo.fuel=readUB2(handle);
    str[sid].cargo.tri=readUB2(handle);
    str[sid].cargo.dur=readUB2(handle);
    str[sid].cargo.mol=readUB2(handle);
    str[sid].cargo.supplies=readUB2(handle);
    str[sid].toplanet.fuel=readUB2(handle);
    str[sid].toplanet.tri=readUB2(handle);
    str[sid].toplanet.dur=readUB2(handle);
    str[sid].toplanet.mol=readUB2(handle);
    str[sid].toplanet.clans=readUB2(handle);
    str[sid].toplanet.supplies=readUB2(handle);
    str[sid].wPID=readUB2(handle);
    str[sid].toship.fuel=readUB2(handle);
    str[sid].toship.tri=readUB2(handle);
    str[sid].toship.dur=readUB2(handle);
    str[sid].toship.mol=readUB2(handle);
    str[sid].toship.clans=readUB2(handle);
    str[sid].toship.supplies=readUB2(handle);
    str[sid].wSID=readUB2(handle);
    str[sid].wIntercept=readUB2(handle);
    str[sid].cargo.money=readUB2(handle);

    if (extraship) {
      str[snum].hullspec=xhullspec[str[snum].wShipType];
      str[snum].engspec=xengspec[str[snum].wDriveLevel];
      if (str[snum].wBeamWeapons > 0) {
        str[snum].beamspec=xbeamspec[str[snum].wBeamLevel];
      }
      if (str[snum].wTorpsBays > 0) {
        str[snum].torpspec=xtorpspec[str[snum].wTorpLevel];
      }
    }
  }

  if (extraship) {
    delete[] xhullspec;
    delete[] xengspec;
    delete[] xbeamspec;
    delete[] xtorpspec;
  }

  fclose(handle);
  return str;
}
